@@ -11,7 +11,7 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 package Dpkg::Source::Package::V3::Native;
 
@@ -26,7 +26,7 @@ use Dpkg;
 use Dpkg::Gettext;
 use Dpkg::ErrorHandling;
 use Dpkg::Compression;
-use Dpkg::Exit;
+use Dpkg::Exit qw(push_exit_handler pop_exit_handler);
 use Dpkg::Version;
 use Dpkg::Source::Archive;
 use Dpkg::Source::Functions qw(erasedir);
@@ -47,8 +47,9 @@ sub do_extract {
     my $basenamerev = $self->get_basename(1);
 
     my $tarfile;
+    my $comp_ext_regex = compression_get_file_extension_regex();
     foreach my $file ($self->get_files()) {
-	if ($file =~ /^\Q$basenamerev\E\.tar\.$compression_re_file_ext$/) {
+	if ($file =~ /^\Q$basenamerev\E\.tar\.$comp_ext_regex$/) {
             error(_g('multiple tarfiles in v1.0 source package')) if $tarfile;
             $tarfile = $file;
 	} else {
@@ -92,7 +93,7 @@ sub do_build {
 
     my ($ntfh, $newtar) = tempfile("$tarname.new.XXXXXX",
                                    DIR => getcwd(), UNLINK => 0);
-    push @Dpkg::Exit::handlers, sub { unlink($newtar) };
+    push_exit_handler(sub { unlink($newtar) });
 
     my ($dirname, $dirbase) = fileparse($dir);
     my $tar = Dpkg::Source::Archive->new(filename => $newtar,
@@ -101,12 +102,12 @@ sub do_build {
     $tar->create(options => \@tar_ignore, chdir => $dirbase);
     $tar->add_directory($dirname);
     $tar->finish();
-    rename($newtar, $tarname) ||
-        syserr(_g("unable to rename `%s' (newly created) to `%s'"),
-               $newtar, $tarname);
-    pop @Dpkg::Exit::handlers;
-    chmod(0666 &~ umask(), $tarname) ||
-        syserr(_g("unable to change permission of `%s'"), $tarname);
+    rename($newtar, $tarname)
+        or syserr(_g("unable to rename `%s' (newly created) to `%s'"),
+                  $newtar, $tarname);
+    pop_exit_handler();
+    chmod(0666 &~ umask(), $tarname)
+        or syserr(_g("unable to change permission of `%s'"), $tarname);
 
     $self->add_file($tarname);
 }
